/* --------------------------------------------------------------------------
 *
 * ETK++ --- The Easy Toolkit for C++ programing
 * Copyright (C) 2004-2006, Anthony Lee, All Rights Reserved
 *
 * $COPYING
 *
 * File: Looper.h
 * Description: Looper for waiting/dispatching message
 *
 * --------------------------------------------------------------------------*/

#ifndef __ETK_LOOPER_H__
#define __ETK_LOOPER_H__

#include <etk/support/List.h>
#include <etk/kernel/OS.h>
#include <etk/app/AppDefs.h>
#include <etk/app/Handler.h>
#include <etk/app/MessageQueue.h>
#include <etk/app/MessageFilter.h>

#ifdef __cplusplus /* Just for C++ */

class EApplication;
class EMessenger;

class _IMPEXP_ETK ELooper : public EHandler {
public:
	ELooper(const char *name = NULL,
		eint32 priority = E_NORMAL_PRIORITY);
	virtual ~ELooper();

	// Archiving
	ELooper(EMessage *from);
	virtual e_status_t Archive(EMessage *into, bool deep = true) const;
	static EArchivable *Instantiate(EMessage *from);

	void		AddHandler(EHandler *handler);
	bool		RemoveHandler(EHandler *handler);
	eint32		CountHandlers() const;
	EHandler	*HandlerAt(eint32 index) const;
	eint32		IndexOf(EHandler *handler) const;

	EHandler	*PreferredHandler() const;
	void		SetPreferredHandler(EHandler *handler);

	bool		IsRunning() const;
	virtual void	*Run();
	virtual void	Quit();
	virtual bool	QuitRequested();
	ELooper*	Proxy() const;
	bool		ProxyBy(ELooper *proxy);

	e_thread_id	Thread() const;

	bool		Lock();
	void		Unlock();
	e_status_t	LockWithTimeout(e_bigtime_t microseconds_timeout);

	eint64		CountLocks() const;
	bool		IsLockedByCurrentThread() const;

	virtual void	DispatchMessage(EMessage *msg, EHandler *target);

	// Empty functions BEGIN --- just for derivative class
	virtual void	MessageReceived(EMessage *msg);
	// Empty functions END

	EMessage	*CurrentMessage() const;
	EMessage	*DetachCurrentMessage();
	EMessageQueue	*MessageQueue() const;

	e_status_t	PostMessage(euint32 command);
	e_status_t	PostMessage(const EMessage *message);
	e_status_t	PostMessage(euint32 command,
				    EHandler *handler,
				    EHandler *reply_to = NULL);
	e_status_t	PostMessage(const EMessage *message,
				    EHandler *handler,
				    EHandler *reply_to = NULL);

	virtual	bool	AddCommonFilter(EMessageFilter *filter);
	virtual	bool	RemoveCommonFilter(EMessageFilter *filter);
	virtual bool	SetCommonFilterList(const EList *filterList);
	const EList	*CommonFilterList() const;

	static ELooper	*LooperForThread(e_thread_id tid);

protected:
	// NextLooperMessage & DispatchLooperMessage: called from task of looper, like below
	//	while(true)
	//	{
	//		...
	//		EMessage *aMsg = NextLooperMessage(E_INFINITE_TIMEOUT);
	//
	//		if(aMsg == NULL) /* after "this->QuitRequested()" return "true" or proxy deconstructing. */
	//		{
	//			...
	//			break;
	//		}
	//		else
	//		{
	//			/* leaked memory unless "DispatchLooperMessage" applied or "delete" instead */
	//			DispatchLooperMessage(aMsg);
	//		}
	//		...
	//	}
	EMessage	*NextLooperMessage(e_bigtime_t timeout = E_INFINITE_TIMEOUT);
	void		DispatchLooperMessage(EMessage *msg);

private:
	friend class EHandler;
	friend class EApplication;
	friend class EMessenger;
	friend e_status_t etk_lock_looper_of_handler(euint64 token, e_bigtime_t timeout);

	bool fDeconstructing;
	ELooper *fProxy;
	EList fClients;

	eint32 fThreadPriority;

	EList fHandlers;
	EHandler *fPreferredHandler;

	void *fLocker;
	eint64 fLocksCount;

	void *fThread;
	void *fSem;

	EMessageQueue *fMessageQueue;
	EMessage *fCurrentMessage;

	static e_status_t _task_(void*);
	static e_status_t _taskLooper_(ELooper*, void*);
	static void _taskError_(void*);

	static EList sLooperList;

	EHandler *_MessageTarget(const EMessage *msg, bool *preferred);
	e_status_t _PostMessage(const EMessage *msg, euint64 handlerToken, euint64 replyToken, e_bigtime_t timeout);

	ELooper *_Proxy() const;
	bool _ProxyBy(ELooper *proxy);
	ELooper *_GetNextClient(ELooper *client) const;

	bool *fThreadExited;

	EList fCommonFilters;
	void _FilterAndDispatchMessage(EMessage *msg, EHandler *target);
};

#endif /* __cplusplus */

#endif /* __ETK_LOOPER_H__ */


